from django import forms
from django.utils.translation import ugettext as _
from django.utils import simplejson

import requests
from flexible_content.forms import BaseItemForm

from .models import Video


class VideoForm(BaseItemForm):
    class Meta(BaseItemForm.Meta):
        model = Video

    def clean_video_id(self):
        """
        Ensure that, for the given service, the video_id is valid.
        """
        failed = False
        d = self.cleaned_data
        service = d.get('service')
        # Get the video id and clear whitespace on either side.
        video_id = d.get('video_id', '').strip()

        # Validate using YouTube's API:
        if service == 'youtube':
            url = ('http://gdata.youtube.com/feeds/api/videos/{}?alt=json'.
                   format(video_id))
            data = requests.get(url)
            # Ensure we can parse the JSON data.
            try:
                json = simplejson.loads(data.text)
            # If not, mark this as a failure.
            except ValueError:
                failed = True

        # Validate using Vimeo's API:
        elif service == 'vimeo':
            data = requests.get('http://vimeo.com/api/v2/video/{}.json'.
                                format(video_id))
            # Ensure we can parse the JSON data.
            try:
                json = simplejson.loads(data.text)
            # If not, mark this as a failure.
            except ValueError:
                failed = True

        # Respond based on the outcome.
        if failed:
            message = _("Couldn't validate video id using {} API. Please "
                        "verify it exists and check for "
                        "typos.".format(service))
            raise forms.ValidationError(message)

        return video_id




from django.conf import settings
from django.conf.urls import patterns, include, url
from django.conf.urls.static import static
from django.contrib.staticfiles.urls import staticfiles_urlpatterns

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    # Examples:
    # url(r'^$', 'mock_project.views.home', name='home'),
    # url(r'^mock_project/', include('mock_project.foo.urls')),

    # Uncomment the admin/doc line below to enable admin documentation:
    # url(r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    url(r'^admin/', include(admin.site.urls)),
)
# Serve static files
urlpatterns += staticfiles_urlpatterns()
urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



from django.db import models
from django.utils.translation import ugettext as _

from flexible_content.models import BaseItem
from flexible_content.utils import get_app_settings


class PlainText(BaseItem):
    text = models.TextField()

    class FlexibleContentInfo:
        description = _("This text won't be interpreted, other than to add "
                        "paragraph/line breaks. In other words, you can't put "
                        "HTML here.")
        name = "Plain Text"
        type_slug = 'plain-text'

    class Meta:
        verbose_name = _("Plain Text")


class RawHTML(BaseItem):
    html = models.TextField()

    class FlexibleContentInfo:
        description = _("Insert custom scripts or snippets of HTML here. Be "
                        "careful, though: you *can* break the site this way.")

    class Meta:
        verbose_name = _("Raw HTML")


class Image(BaseItem):
    uploaded_file = models.FileField(upload_to='flexible-content/images')

    class FlexibleContentInfo:
        description = _("Upload an image and have it displayed as the site "
                        "chooses. You can also specify a heading or caption. "
                        "Note that it's usually best to put an image *before* "
                        "any text it should appear alongside.")

    class Meta:
        verbose_name = _("Image")


class Download(BaseItem):
    uploaded_file = models.FileField(upload_to='flexible-content/downloads')

    class FlexibleContentInfo:
        description = _("Upload a file and have it advertised as the site "
                        "chooses. You can also specify a heading or caption. "
                        "Note that it's usually best to put a download "
                        "*before* any text it should appear alongside.")

    class Meta:
        verbose_name = _("Download")


def get_default_video_service():
    settings = get_app_settings()
    return settings.get('DEFAULT_VIDEO_SERVICE', '')


class Video(BaseItem):
    SERVICE_CHOICES = (
        ('youtube', _('YouTube')),
        ('vimeo', _('Vimeo')),
    )
    # HELP TEXT (for fields)
    HELP_VIDEO_ID = _("The number/code/identifier for your video. You can "
                      "usually find it in the URL, but each video service is "
                      "different.")

    service = models.CharField(max_length=25,
                               default=lambda: get_default_video_service())
    video_id = models.CharField(verbose_name=_("Video ID"), max_length=100,
                                help_text=HELP_VIDEO_ID)

    class FlexibleContentInfo:
        description = _("Give details about a video and have it displayed as "
                        "the site chooses. You can also specify a heading or "
                        "caption. Note that it's usually best to put an video "
                        "*before* any text it should appear alongside.")

    class Meta:
        verbose_name = _("Video")

    def get_form_class(self):
        from .forms import VideoForm
        return VideoForm


# Register all of the types below
DEFAULT_TYPES = (PlainText, RawHTML, Image, Download, Video)



from django.core.exceptions import ImproperlyConfigured
from django.contrib.admin.sites import AdminSite
from django.contrib.contenttypes.models import ContentType
from django.test import SimpleTestCase, TestCase
from django.test.client import Client, RequestFactory
from django.test.utils import override_settings

from mock_project.test_app.models import MyArea, MyItem

from .admin import ContentAreaAdmin, FORM_PREFIX_PLACEHOLDER, get_form_prefix
from .models import BaseItem, ContentArea, TemporaryArea
from .default_item_types.models import (DEFAULT_TYPES, PlainText, RawHTML,
                                        Image, Download, Video)
from .utils import get_app_settings, get_models_from_strings


CUSTOM_TYPES_STRING = (
    'default_item_types.PlainText',
    'default_item_types.Video',
    'test_app.MyItem',
)
CUSTOM_TYPES_CLASSES = (
    PlainText,
    Video,
    MyItem,
)


class TestDataMixin(object):
    # Load in a basic area with two types.
    fixtures = ['test-data.json']
    data_for_another_area = {
        'title': "New Page with Two Item",
        'fc-prefixes': 'fc-item-1,fc-item-2',
        # ITEM 1: Leave all non-required fields blank to make sure they're
        # robustly overwritten.
        'fc-item-1-content_area_ct': '',
        'fc-item-1-content_area_id': '',
        'fc-item-1-ct': ContentType.objects.get_for_model(PlainText).pk,
        'fc-item-1-pk': '',
        'fc-item-1-delete': 0,
        'fc-item-1-ordering': 2,
        'fc-item-1-text': "Blah blah blah.\n\nYippee.",
        # ITEM 2: Leave all non-required fields out to make sure they're
        # added in.
        'fc-item-2-ordering': 1,
        'fc-item-2-delete': 0,
        'fc-item-2-ct': ContentType.objects.get_for_model(RawHTML).pk,
        'fc-item-2-html': "<p>Testing!&amp;</p>",
    }
    data_for_updating_first_area = {
        'title': "Our original area, but updated",
        'fc-prefixes': 'fc-item-1,fc-item-2,fc-item-3,fc-item-4',

        # ITEM 1: Update the text and swap its place with item 2. Leave the
        # content area and stuff blank, to make sure it overwrites them.
        'fc-item-1-content_area_ct': '',
        'fc-item-1-content_area_id': '',
        'fc-item-1-pk': 1,  # Must stay consistent with our JSON fixture
        'fc-item-1-ct': ContentType.objects.get_for_model(PlainText).pk,
        'fc-item-1-ordering': 2,
        'fc-item-1-delete': 0,
        'fc-item-1-text': "Blah blah blah.\n\nYippee.",

        # ITEM 2: Delete it! And leave all non-required fields out to make
        # sure they're added in.
        'fc-item-2-ordering': 1,
        'fc-item-2-delete': 1,
        'fc-item-2-ct': ContentType.objects.get_for_model(Video).pk,
        'fc-item-2-pk': 2,  # Must stay consistent with our JSON fixture
        'fc-item-2-service': "vimeo",
        'fc-item-2-video_id': "59338758",

        # ITEM 3: An additional item! But delete it right away.
        'fc-item-3-ordering': 3,
        'fc-item-3-delete': 1,
        'fc-item-3-ct': ContentType.objects.get_for_model(RawHTML).pk,
        'fc-item-3-html': "<p>lkajsdlkfj alksjdfaklsf</p>",

        # ITEM 4: Add another item, but this time keep it!
        'fc-item-4-ordering': 4,
        'fc-item-4-delete': 0,
        'fc-item-4-ct': ContentType.objects.get_for_model(PlainText).pk,
        'fc-item-4-text': "blah blah blah blah new item",
    }
    invalid_data_for_updating_first_area = {
        'title': "Our original area, but updated with bad info; should fail",
        'fc-prefixes': 'fc-item-1,fc-item-2',

        'fc-item-1-pk': 1,  # Must stay consistent with our JSON fixture
        'fc-item-1-ct': ContentType.objects.get_for_model(PlainText).pk,
        'fc-item-1-ordering': 1,
        'fc-item-1-delete': 0,
        'fc-item-1-text': "",  # This field can't be blank!

        'fc-item-2-ordering': 2,
        'fc-item-2-delete': 0,
        'fc-item-2-ct': ContentType.objects.get_for_model(Video).pk,
        'fc-item-2-pk': 2,  # Must stay consistent with our JSON fixture
        'fc-item-2-service': "vimeo",
        'fc-item-2-video_id': "59338758",
    }

    @property
    def area(self):
        return MyArea.objects.get(pk=1)

    @property
    def item_1(self):
        return PlainText.objects.get(pk=1)

    @property
    def item_2(self):
        return Video.objects.get(pk=2)


class ConfiguredTypesTest(SimpleTestCase):
    """
    Make sure the app reports the proper types as being available, for any
    given configuration.
    """

    @override_settings(FLEXIBLE_CONTENT=None)
    def test_without_settings(self):
        """
        Use blank settings to make sure it recovers by using the default types.
        """
        from .default_item_types.models import DEFAULT_TYPES
        # Make sure it gave us the default types.
        types = BaseItem.get_configured_types()
        self.assertEqual(types, DEFAULT_TYPES, "Configuration had no types "
                         "specified, but it didn't recover by falling back "
                         "to the default item types.")

    @override_settings(FLEXIBLE_CONTENT="Hi!")
    def test_with_bad_settings(self):
        """
        Use invalid settings to make sure it complains about it.
        """
        with self.assertRaises(ImproperlyConfigured):
            BaseItem.get_configured_types()

    @override_settings(FLEXIBLE_CONTENT={'ITEM_TYPES': ('fake.DoesNotExist',)})
    def test_with_nonexistent_type(self):
        """
        Give it a nonexistent app and model to make sure it complains.
        """
        with self.assertRaises(ImportError):
            BaseItem.get_configured_types()

    @override_settings(FLEXIBLE_CONTENT={'ITEM_TYPES': CUSTOM_TYPES_STRING})
    def test_with_custom_types(self):
        """
        Configure the project's settings to use the custom type strings above.
        """
        types = BaseItem.get_configured_types()
        # Make sure it matches our tuple of those classes.
        self.assertEqual(types, CUSTOM_TYPES_CLASSES, "Configuration didn't "
                         "take our custom classes properly.")


@override_settings(FLEXIBLE_CONTENT={'ITEM_TYPES': CUSTOM_TYPES_STRING})
class AreaTest(TestCase):
    """
    Ensure that all of the basic functions work, at least somewhat.
    """

    def setUp(self):
        # Create an area with regular ordering (first item first).
        self.area_a = MyArea.objects.create(title="Blah")
        self.item_a1 = MyItem.objects.create(ordering=1,
                                             content_area=self.area_a,
                                             my_number=81)
        self.item_a2 = PlainText.objects.create(ordering=2,
                                                content_area=self.area_a,
                                                text="blarg! blalkdjf HI!")

        # Create another area with flipped items - create the second one first,
        # to make sure their ordering number takes precedence over their PK.
        self.area_b = MyArea.objects.create(title="Foo")
        self.item_b2 = RawHTML.objects.create(ordering=2,
                                              content_area=self.area_b,
                                              html="<p>Blah blah blah.</p>")
        self.item_b1 = Video.objects.create(ordering=1,
                                            content_area=self.area_b,
                                            service='vimeo',
                                            video_id="59338758")

    def test_get_items_via_area(self):
        """
        Areas should get a list of their items, and in the right order.
        """
        # Use the content_items property (actually a method) to get area A's
        # items. Listify them, so we can do an assertEqual.
        items = list(self.area_a.items)
        expected = [self.item_a1, self.item_a2]

        self.assertEqual(items, expected,
                         "We didn't get back the items from the area's "
                         "property as expected.\nGot: {}\nExpected: {}".
                         format(items, expected))

    def test_get_items_via_manager(self):
        """
        Areas should get a list of their items, and in the right order.
        """
        # Use the BaseItem manager to fetch the area's items. Listify them,
        # so we can do an assertEqual.
        items = list(BaseItem.objects.get_for_area(self.area_b))
        expected = [self.item_b1, self.item_b2]

        self.assertEqual(items, expected,
                         "We didn't get the items from the manager like we "
                         "expected to.\nGot: {}\nExpected: {}".
                         format(items, expected))

    def test_get_rendered_content(self):
        """
        Areas should be able to get their HTML on demand, and the result
        should contain the data from the models.
        """
        content_a = self.area_a.get_rendered_content()

        # Make sure the content from both items appears somewhere in the
        # output!
        self.assertIn(str(self.item_a1.my_number), content_a,
                      "Couldn't find first item's my_number ({}) in rendered "
                      "content.".format(self.item_a1.my_number))
        self.assertIn(self.item_a2.text, content_a,
                      "Couldn't find second item's text ({}) in rendered "
                      "content.".format(self.item_a2.text))

    def test_items_deleted_with_area(self):
        """
        When we delete an area, its items should be cleared with it.
        """

        # Store the PKs we'll check for later.
        item_pks = [i.pk for i in self.area_a.items]
        # Make sure we have items, or this test won't work.
        self.assertGreater(len(item_pks), 0, "The area must have at least one "
                           "item for this test to work.")

        # Delete the area and make sure its items are gone.
        self.area_a.delete()
        self.assertEqual(BaseItem.objects.filter(pk__in=item_pks).count(), 0,
                         "We deleted an area, but its items remained.")


@override_settings(FLEXIBLE_CONTENT={'ITEM_TYPES': CUSTOM_TYPES_STRING})
class ItemTest(TestDataMixin, TestCase):
    """
    Make sure we can create, edit, and delete items of different types.
    """

    def setUp(self):
        pass

    def test_get_form_prefix_default(self):
        self.assertEqual(get_form_prefix(), 'fc-item-PLACEHOLDER',
                         "Didn't get the form prefix default like expected.")

    def test_get_form_prefix_counter(self):
        self.assertEqual(get_form_prefix(counter=1), 'fc-item-1',
                         "Didn't get the form prefix with the counter we "
                         "provided.")

    def test_get_form_template(self):
        """
        Make sure we can get the HTML for an item.
        """
        html = self.item_1.get_form_template().as_content_item()

        self.assertIn(FORM_PREFIX_PLACEHOLDER, html,
                      "The form template doesn't include the placeholder "
                      "text.")

    def test_get_casted(self):
        """
        Make sure we can cast a basic BaseItem instance.
        """
        item = BaseItem.objects.all()[0]
        self.assertEqual(type(item), BaseItem,
                         "We queried for a BaseItem but got the casted "
                         "version back unnecessarily.")

        casted_item = item.get_casted()
        self.assertNotEqual(type(casted_item), BaseItem,
                            "We asked for the casted version of an item, but "
                            "we still got a BaseItem back.")

    def test_get_type_description(self):
        """
        Make sure that we can get a type's description.
        """
        self.assertEqual(self.item_1.get_type_description(),
                         self.item_1.__class__.FlexibleContentInfo.description,
                         "We asked for an item's description, but it didn't "
                         "match what was defined in its FlexibleContentInfo "
                         "class.")

    def test_create_simple_type(self):
        """
        Can we create an instance of an item without a defined form?
        """
        # Create a blank item.
        new_item = MyItem()

        # Create some fake data to submit.
        fake_POST = {
            'content_area_id': self.area.pk,
            'content_area_ct': self.area.get_content_type().pk,
            'ordering': 3,
            'my_number': 6090,
        }

        # Set up the form with this data and save it!
        form_with_data = new_item.get_form(data=fake_POST)
        form_with_data.save()

        # See if the form has an instance with valid primary key.
        if not getattr(form_with_data.instance, 'pk'):
            self.fail("Couldn't create a MyItem by saving faked data through "
                      "a form.")

    def test_create_type_with_custom_form(self):
        """
        Can we create an instance of an item that uses a custom form?
        """
        new_item = Video()

        # Create a form instance without a saved instance.
        initial_form = new_item.get_form()

        # Ensure the form has the video-ish fields on it.
        self.assertIn('video_id', initial_form.fields,
                      "Couldn't get a form instance from an unsaved item.")

        # Re-render the form and supply data.
        fake_POST = {
            'content_area_id': self.area.pk,
            'content_area_ct': self.area.get_content_type().pk,
            'ordering': 3,
            'service': 'vimeo',
            'video_id': "59338758",
        }

        form_with_data = new_item.get_form(data=fake_POST)
        # Save it!
        form_with_data.save()

        # See if the form has an instance with valid primary key.
        if not getattr(form_with_data.instance, 'pk'):
            self.fail("Couldn't create a video by saving faked data through "
                      "a form.")

    def test_get_instance_form(self):
        """
        Can we get the form for an existing content item?
        """
        # Ensure that we can get the form and that its instance matches
        # the one we used to get the form.
        self.assertEqual(self.item_1.get_form().instance, self.item_1,
                         "We used an item to generate a bound form, but the "
                         "instance bound to that form didn't match our item!")

    def test_get_type_name(self):
        """
        Can we use a specified type name, or generate one automatically?
        """

        # First, a custom name.
        result = PlainText().get_type_name()
        expected = "Plain Text"
        # For a class that specifies a type name, do we get what we expect?
        self.assertEqual(result, expected,
                         "The type name wasn't returned as expected. Got {} "
                         "instead of {}.".format(result, expected))

        # Now, an automatically generated one.
        result = Video().get_type_name()
        expected = "Video"
        # What about a type that doesn't specify a name?
        self.assertEqual(result, expected,
                         "The type name wasn't generated as expected. Got {} "
                         "instead of {}.".format(result, expected))

    def test_get_type_slug(self):
        """
        Can we use a specified type slug, or generate one automatically?
        """

        # For a class that specifies a type slug, do we get what we expect?
        result = PlainText().get_type_slug()
        expected = 'plain-text'
        self.assertEqual(result, expected,
                         "The type slug wasn't returned as expected. Got {} "
                         "instead of {}.".format(result, expected))

        # Now, an automatically generated one.
        result = Video().get_type_slug()
        expected = 'video'
        # What about a type that doesn't specify a name?
        self.assertEqual(result, expected,
                         "The type slug wasn't generated as expected. Got {} "
                         "instead of {}.".format(result, expected))

    def test_get_template_name(self):
        """
        Make sure that we can get a type's template name/path.
        """
        self.assertEqual(self.item_2.get_template_name(),
                         'flexible-content/video.html',
                         "We didn't get the template name we expected.")

    def test_delete_base_item(self):
        """
        Ensure that the subclass is deleted too, for generic FK integrity.
        """
        # Get the first item
        base_item = BaseItem.objects.all()[:1][0]
        casted_item = base_item.get_casted()

        # store the pk and model we'll query
        pk = int(casted_item.pk)
        model = casted_item.__class__

        # Delete the base item, then make sure its subclass isn't not still
        # there.
        base_item.delete()
        self.assertEqual(model.objects.filter(pk=pk).count(), 0,
                         "We deleted a BaseItem specifically, but the record "
                         "of its subclass instance remained.")

    def test_delete_item_subclass(self):
        """
        Ensure that the base item is deleted too, for generic FK integrity.
        """
        # Get the first item
        casted_item = BaseItem.objects.all().select_subclasses()[:1][0]

        # store the pk
        pk = int(casted_item.pk)

        # Delete the subclass/casted item and make sure the base item is gone
        # too.
        casted_item.delete()
        self.assertEqual(BaseItem.objects.filter(pk=pk).count(), 0,
                         "We deleted an item, but its BaseItem remained.")


class AdminUnitTest(TestDataMixin, TestCase):
    # Test the custom admin methods to make sure they handle input properly.

    def setUp(self):
        self.factory = RequestFactory()
        self.site = AdminSite()
        self.admin = ContentAreaAdmin(MyArea, self.site)

    def test_fc_get_context(self):
        # Make sure that, when we're just loading a page, it gives us the
        # proper context.
        request = self.factory.get('/admin/test_app/myarea/{}/'.
                                   format(self.area.pk))

        # Call the method!
        context = self.admin.fc_get_context(request, obj=self.area)

        # FC_TYPES
        # Test the number of types being sent to the template.
        self.assertEqual(len(context['fc_types']),
                         len(BaseItem.get_configured_types()))
        # Ensure we're getting an instance of the model.
        self.assertEqual(context['fc_types'][0].__class__,
                         BaseItem.get_configured_types()[0])

        # FC_FORMS
        # Are we getting the right number of forms for this area?
        self.assertEqual(len(context['fc_forms']), 2)
        # Make sure the first form's instance is the first item on this area.
        self.assertEqual(context['fc_forms'][0].instance,
                         self.area.items[0])

        # the prefix
        self.assertEqual(context['fc_form_prefix_placeholder'],
                         FORM_PREFIX_PLACEHOLDER)

    def test_fc_get_form_model_by_prefix(self):
        # For a given item in the POST vars, we should be able to load the
        # corresponding model.

        # Generate a fake request object that we can pass to the method.
        plain_text_ct = ContentType.objects.get_for_model(PlainText)
        request = self.factory.post('/admin/test_app/myarea/add/',
                                    {'fc-item-1-ct': plain_text_ct.pk})
        # Get the model for that item.
        model = self.admin.fc_get_form_model_by_prefix(request, 'fc-item-1')

        # Ensure the admin it gave us was the one we wanted.
        self.assertEqual(model, PlainText)

    def test_fc_get_forms_from_POST(self):
        # Make sure fc_get_forms_from_POST can load unbound forms.

        # Start by creating a new a new area ----------------------------------
        data = self.data_for_another_area
        request = self.factory.post('/admin/test_app/myarea/add/', data)

        # Get the forms based on that request.
        forms = self.admin.fc_get_forms_from_POST(request)

        # Make sure everything's hunkey-dorey!

        # Ensure neither form has an instance.
        self.assertEqual(forms[0].instance.pk, None)
        self.assertEqual(forms[1].instance.pk, None)

        # Ensure that the forms have been passed the right POST data.
        self.assertEqual(forms[0].data['fc-item-1-text'],
                         data['fc-item-1-text'])
        self.assertEqual(forms[1].data['fc-item-2-html'],
                         data['fc-item-2-html'])

        # Now make sure we can update an existing item ------------------------
        url = '/admin/test_app/myarea/{}'.format(self.area.pk)
        data = self.data_for_updating_first_area
        request = self.factory.post(url, data)

        # Get the forms based on that request.
        forms = self.admin.fc_get_forms_from_POST(request)

        # Ensure the length.
        self.assertEqual(len(forms), 4)

        # We should find that the second two items are about to be deleted,
        # but not the first.
        self.assertEqual(forms[0].data['fc-item-1-delete'], '0')
        self.assertEqual(forms[1].data['fc-item-2-delete'], '1')
        self.assertEqual(forms[2].data['fc-item-3-delete'], '1')
        self.assertEqual(forms[3].data['fc-item-4-delete'], '0')

        # Make sure a basic, type-specific piece of info came through with
        # each.
        self.assertEqual(forms[0].data['fc-item-1-text'],
                         data['fc-item-1-text'])
        self.assertEqual(forms[1].data['fc-item-2-video_id'],
                         data['fc-item-2-video_id'])
        self.assertEqual(forms[2].data['fc-item-3-html'],
                         data['fc-item-3-html'])
        self.assertEqual(forms[3].data['fc-item-4-text'],
                         data['fc-item-4-text'])

    def test_fc_get_forms_GET_add(self):
        self.fail("You haven't finished this test yet.")

    def test_fc_get_forms_GET_change(self):
        """
        Forms for existing items should always be bound and have a PK.
        """
        request = self.factory.get('/admin/test_app/myarea/{}/'.
                                   format(self.area.pk))

        forms = self.admin.fc_get_forms(request, obj=self.area)

        # Ensure that each of the existing items is still bound to the proper
        # form.
        existing_item_pks = [i.pk for i in self.area.items]
        for f in forms:
            pk = getattr(f.instance, 'pk', None)
            # If it has a PK, remove that PK from our checklist
            if pk is not None:
                existing_item_pks.remove(pk)

        # Are there any existing items that weren't found in the forms?
        self.assertEqual(len(existing_item_pks), 0)

    def test_fc_get_forms_POST_change(self):
        """
        Forms for existing items should always be bound and have a PK.
        """
        url = '/admin/test_app/myarea/{}/'.format(self.area.pk)
        data = self.data_for_updating_first_area
        request = self.factory.post(url, data)

        forms = self.admin.fc_get_forms(request, obj=self.area)

        # Ensure that each of the existing items is still bound to the proper
        # form.
        existing_item_pks = [i.pk for i in self.area.items]
        for f in forms:
            pk = getattr(f.instance, 'pk', None)
            # If it has a PK, remove that PK from our checklist
            if pk is not None:
                existing_item_pks.remove(pk)

        # Are there any existing items that weren't found in the forms?
        self.assertEqual(len(existing_item_pks), 0)

    def test_save_model(self):
        self.fail("You haven't finished this test yet.")


class AdminIntegrationTest(TestDataMixin, TestCase):
    # Ensure that requests can be made as expected.

    def setUp(self):
        self.client = Client()
        self.client.login(username='test', password='test')

    def test_create_itemless_area(self):
        # Ensure that we can create an area without content items (edge case).

        # What page will we hit?
        url = '/admin/test_app/myarea/add/'
        data = {
            'title': "Test Page - no items",
            'fc-prefixes': '',
        }

        # Get the areas that already exist.
        existing_area_pks = tuple(MyArea.objects.values_list('pk', flat=True))

        # Hit the admin with our 'create new area' request.
        response = self.client.post(url, data)

        # Get newly-created areas. There should only be one, and it should
        # have the title we're looking for.
        new_areas = tuple(MyArea.objects.exclude(pk__in=existing_area_pks))

        # Make sure all is well.
        self.assertEqual(len(new_areas), 1)
        self.assertEqual(new_areas[0].title, data['title'])

    def test_create_area_with_items(self):
        # Ensure that we can create an area a few items.

        # What page will we hit?
        url = '/admin/test_app/myarea/add/'
        data = self.data_for_another_area

        # Get the areas that already exist.
        existing_area_pks = tuple(MyArea.objects.values_list('pk', flat=True))
        existing_item_pks = tuple(BaseItem.objects.
                                  values_list('pk', flat=True))

        # Hit the admin with our 'create new area' request.
        response = self.client.post(url, data, follow=True)

        # Get newly-created areas. Make sure there's only one and that its data
        # is correct.
        new_areas = tuple(MyArea.objects.exclude(pk__in=existing_area_pks))
        self.assertEqual(len(new_areas), 1)
        self.assertEqual(new_areas[0].title, data['title'])

        # Get the new items. Are there two, and is their data accurate?
        new_items = tuple(new_areas[0].items.exclude(pk__in=existing_item_pks))
        self.assertEqual(len(new_items), 2)
        # We should get the HTML first, since we put them in in the reverse
        # order of their ordering number.
        self.assertEqual(new_items[0].html, data['fc-item-2-html'])
        self.assertEqual(new_items[1].text, data['fc-item-1-text'])

    def test_update_area_delete_item(self):
        # Ensure that we can update an existing area and delete one item.

        this_area_pk = self.area.pk
        url = '/admin/test_app/myarea/{}/'.format(this_area_pk)
        data = self.data_for_updating_first_area

        # Get the areas that already exist.
        self.assertEqual(len(self.area.items), 2)

        # Hit the admin with our 'update this area' request.
        response = self.client.post(url, data)

        # Make sure our area is still there.
        try:
            area = MyArea.objects.get(pk=this_area_pk)
        except MyArea.DoesNotExist:
            self.fail("For some reason, the area was deleted when trying to "
                      "delete one of its items.")
        self.assertEqual(area.title, data['title'])

        # Make sure we still have two items. The second should have been
        # deleted, another one never completed (marked as deleted before saved)
        # and another that was genuinely added. So combined with the one that
        # was merely updated, that makes two.
        self.assertEqual(len(self.area.items), 2)

        # Make sure the first one has the updated text, and still has the same
        # primary key.
        first_item = area.items[0]
        self.assertEqual(first_item.text, data['fc-item-1-text'])
        self.assertEqual(first_item.pk, data['fc-item-1-pk'])

        # Make sure the new item (with ordering four; the second remaining)
        # was saved and has the right ContentType and everything.
        second_item = area.items[1]
        self.assertEqual(second_item.text, data['fc-item-4-text'])
        self.assertEqual(second_item.get_content_type().pk,
                         data['fc-item-4-ct'])
        self.assertEqual(second_item.ordering, data['fc-item-4-ordering'])

    def test_update_area_with_invalid_items(self):
        """
        Does it stop us if one of the items doesn't validate?
        """

        url = '/admin/test_app/myarea/{}/'.format(self.area.pk)
        data = self.invalid_data_for_updating_first_area

        # Hit the admin with our 'update this area' request.
        response = self.client.post(url, data, follow=True)

        self.assertEqual(len(response.redirect_chain), 0, "We submitted bad "
                         "item data, but it still redirected us back to the "
                         "listing page.")



from django import forms
from django.contrib import admin

from flexible_content.admin import ContentAreaAdmin, ContentAreaAdminForm

from .models import MyArea


class MyAreaAdminForm(ContentAreaAdminForm):
    class Meta:
        model = MyArea


class MyAreaAdmin(ContentAreaAdmin):
    form = MyAreaAdminForm


admin.site.register(MyArea, MyAreaAdmin)



from django import forms
from django.template.loader import render_to_string


class BaseItemForm(forms.ModelForm):
    content_item_template = 'flexible-content/_single-item-form.html'

    delete = forms.BooleanField(initial=False, required=False)

    class Meta(object):
        pass

    def already_exists(self):
        """
        Report on whether or not this instance has been saved.
        """
        return bool(getattr(self.instance, 'pk', None))

    def as_content_item(self):
        """
        Instead of as_p, as_ul, and the like, render a content item as
        it shows in the admin.
        """
        return render_to_string(self.content_item_template, {'form': self})

    def get_unique_fields(self):
        """
        Which fields aren't part of BaseItem? These will be presented front
        and center.
        """
        baseitem_fields = (
            'content_area_ct',
            'content_area_id',
            'ordering',
            'delete',
        )

        # Loop through each field:
        unique_fields = []
        for f in self.visible_fields():
            # So long as it's not one of the baseitem fields, add it to our
            # list.
            if f.name not in baseitem_fields:
                unique_fields.append(f)
        return unique_fields

    def should_be_deleted(self):
        """
        Look at the incoming data and figure out whether it's marked for
        deletion.
        """
        return bool(int(self.data.get('{}-delete'.format(self.prefix), '0')))




"""
I am sorry for this file.

You see, Django does have support for inlines. But not inlines of different
types. So these hacks take over the space between `fieldsets` and `inlines`
and manage these content items on their own.

The first time I wrote this, I replaced the add_view and change_view methods
altogether. This is bad because it meant any changes to the stock Django
methods don't propagate to the overridden methods.

This version is a complete rewrite with the goal of using super() instead of
nuking ModelAdmin's add_view and change_view. That means that all the value
they add has to happen before and after the super() call. Consequently, parts
of this are less than obvious, because it's not given a piece of data
explicitly.

Ideally, this would be redone in the future to be more elegant - perhaps
replacing the views altogether, or allowing front-end editing via an API.
"""

from django import forms
from django.contrib import admin
from django.contrib.admin.util import unquote
from django.contrib.contenttypes.models import ContentType
from django.db import models, transaction
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext as _
from django.views.decorators.csrf import csrf_protect

from .models import BaseItem, TemporaryArea


csrf_protect_m = method_decorator(csrf_protect)

FORM_PREFIX_TEMPLATE = 'fc-item-{counter}'
FORM_PREFIX_PLACEHOLDER = 'PLACEHOLDER'


def get_form_prefix(counter=FORM_PREFIX_PLACEHOLDER):
    return FORM_PREFIX_TEMPLATE.format(counter=counter)


class ContentAreaAdminForm(forms.ModelForm):
    """
    Make the area's validation depend on its items' validation.
    """

    # This will be our hook into the area's validation process from add_view
    # and change_view. If we don't mark this field as True, it'll fail with
    # a validation error. Let it be unchecked here, but raise a general (non-
    # field-specific) error in the clean method.
    all_items_validated = forms.BooleanField(required=False,
                                             widget=forms.HiddenInput)

    def clean(self):
        d = self.cleaned_data

        # Get the value they submitted for item validation, cast it to an int
        # (so an accidental '0' string won't pass as true), and evaluate it
        # boolean-ly.
        all_items_validated = bool(int(d.get('all_items_validated', '0')))
        if not all_items_validated:
            message = _("One of the content items didn't validate. Please "
                        "check them and try again.")
            raise forms.ValidationError(message)

        return d


class ContentAreaAdmin(admin.ModelAdmin):
    """
    Allow management of content items from within the admin.

    Since ModelAdmin itself is complex, use 'fc_' as a prefix to all
    attributes that we add here. Hopefully this will make it more clear when
    we're working in here vs. in ModelAdmin or elsewhere.
    """
    change_form_template = 'flexible-content/change-form.html'
    save_on_top = True

    @csrf_protect_m
    @transaction.commit_on_success
    def add_view(self, request, form_url='', extra_context=None):
        """
        Override the add/create view to allow editing and validation of items.
        """

        # Add in the necessary context.
        if extra_context is None:
            extra_context = {}
        extra_context = self.fc_get_context(request)

        # Save the items they submitted.
        temp_area = None
        if request.method == 'POST':
            temp_area = self.fc_save_items(request,
                                           forms=extra_context['fc_forms'])

        # Call ModelAdmin's add_view.
        response = (super(ContentAreaAdmin, self).
                    add_view(request, form_url, extra_context))

        # If the above object was saved, attach our items and save them.
        new_area = getattr(request, 'new_content_area_object', None)
        if temp_area is not None and new_area is not None:
            temp_area.migrate_items_to(new_area)

        return response

    @csrf_protect_m
    @transaction.commit_on_success
    def change_view(self, request, object_id, form_url='', extra_context=None):
        """
        Override the change view to allow editing and validation of items.
        """
        # Redundant super().change_view will do this again, but meh.
        obj = self.get_object(request, unquote(object_id))

        # Add in the necessary context.
        if extra_context is None:
            extra_context = {}
        extra_context = self.fc_get_context(request, obj)

        if request.method == 'POST':
            self.fc_save_items(request, area=obj,
                               forms=extra_context['fc_forms'])

        # Call ModelAdmin's add_view.
        response = (super(ContentAreaAdmin, self).
                    change_view(request, object_id, form_url, extra_context))

        return response

    def fc_save_items(self, request, area=None, forms=None):
        """
        For each form, create/update objects.
        """
        all_items_validated = True

        # If the current area doesn't exist yet, create a temporary one to save
        # the items to.
        if area is None:
            using_temp_area = True
            area = TemporaryArea.objects.create()
        else:
            using_temp_area = False

        # If there are any forms, loop through them.
        if len(forms):
            # We'll need the data on the area we're saving it to.
            area_ct = area.get_content_type().pk
            area_id = area.pk

            for f in forms:

                # If this form's data includes a non-zero value for the delete
                # field, they want it deleted.
                if f.should_be_deleted():
                    # We only need to actively delete it if it's already in
                    # the database.
                    if f.already_exists():
                        f.instance.delete()
                    # Don't do anything else, if they didn't want their changes
                    # saved.
                    continue

                # Ensure that the object we're saving is assigned to the proper
                # area, especially if the real content area doesn't exist yet.
                f.data.update({
                    '{}-content_area_ct'.format(f.prefix): area_ct,
                    '{}-content_area_id'.format(f.prefix): area_id,
                })

                # Save the form.
                try:
                    f.save()
                # Suppress these errors, since the Django admin doesn't catch
                # them like it does for the forms it knows about. We'll address
                # this later.
                except ValueError as e:
                    all_items_validated = False

        # Set the main area's form validation field based on our assessment
        # of the items' validation. If this value is zero, it will trigger
        # the admin to fail validation because we told it items were invalid.
        request.POST = request.POST.copy()
        request.POST.update({
            'all_items_validated': all_items_validated and '1' or '0',
        })

        # Only return something if we're using a temporary area.
        if using_temp_area:
            return area
        # Otherwise, return nothing.
        else:
            return None

    def fc_get_context(self, request, obj=None):
        """
        Add flexible_content context we'll need for add_view and change_view.
        """
        return {
            'fc_types': [t() for t in BaseItem.get_configured_types()],
            'fc_forms': self.fc_get_forms(request, obj=obj),
            'fc_form_prefix_placeholder': FORM_PREFIX_PLACEHOLDER,
        }

    def fc_get_form_model_by_prefix(self, request, prefix):
        # Try to load the model class via the ContentType they suggested.
        ct_pk = request.POST.get('{}-ct'.format(prefix), None)
        try:
            model_class = ContentType.objects.get(pk=ct_pk).model_class()
        # Be ready for a content type not to exist. Don't worry about the
        # AttributeError - the doesnotexist should be raised first.
        except ContentType.DoesNotExist:
            message = ("Form said its type was that of ContentType {}, but "
                       "that ContentType couldn't be found in the "
                       "database.".format(ct_pk))
            raise ValueError(message)

        # If the model isn't a subclass of BaseItem, that's also a problem.
        if not issubclass(model_class, BaseItem):
            message = ("For ContentType {}, we got the model {}, which isn't "
                       "even a subclass of BaseItem.".
                       format(ct_pk, model_class))
            raise Exception(message)

        return model_class

    def fc_get_forms_from_POST(self, request):
        forms = []

        # What prefixes are there?
        prefixes = [p for p in
                    request.POST.get('fc-prefixes', '').split(',') if p]

        # Loop through each submitted item and process its data.
        for p in prefixes:

            # Check the PK for this prefix
            pk = request.POST.get('{}-pk'.format(p), None)
            # If this prefix is for an existing item, load its data.
            instance = None
            if pk:
                try:
                    instance = (BaseItem.objects.filter(pk=pk).
                                select_subclasses()[:1][0])
                except IndexError:
                    message = ("Form reported update of BaseItem with id {}, "
                               "but no such BaseItem exists.".format(pk))
                    raise ValueError(message)

            # We need to copy the POST data so it can be updated (QueryDicts
            # are immutable). Plus, there's less redundant data.

            # Append a hypen to the prefix. Django does this in the form, but
            # we need to prevent getting fc-item-10 when we ask for fc-item-1,
            # and adding the hyphen will avoid that.
            search_prefix = '{}-'.format(p)
            # Assemble a dictionary of only this form's data.
            this_form_data = dict([(k, v) for (k, v) in request.POST.items()
                                   if k.startswith(p)])

            # Alright, it's time to create the form! If we found an instance,
            # use that to create it.
            if isinstance(instance, BaseItem):
                form = instance.get_form(data=this_form_data,
                                         files=request.FILES, prefix=p)
            # Otherwise, we'll have to get the model from the content type
            # field first.
            else:
                model_class = self.fc_get_form_model_by_prefix(request, p)

                form = model_class().get_form(data=this_form_data,
                                              files=request.FILES, prefix=p)

            # Append the form to the end of the list.
            forms.append(form)

        return forms

    def fc_get_forms(self, request, obj=None):
        """
        Use the request to get the forms that should be rendered to the page.
        """
        forms = []

        # Are they POSTing data, or should we merely get it from the DB?
        if request.method == 'POST':
            forms = self.fc_get_forms_from_POST(request)
        # If this is a GET on an existing area.
        elif obj:
            for i in obj.items:
                prefix = get_form_prefix(len(forms) + 1)
                forms.append(i.get_form(prefix=prefix))
        # If this is a GET on an area's ADD page.
        else:
            forms = []

        return forms

    def get_fieldsets(self, request, obj=None):
        """
        Ensure that 'all_items_validated' never shows up in the fieldsets.
        """
        fieldsets = super(ContentAreaAdmin, self).get_fieldsets(request, obj)

        # Loop through the fieldset declaration and remove the tricksy field.
        for fs in fieldsets:
            fs_info = fs[1]
            if 'all_items_validated' in fs_info['fields']:
                fs_info['fields'] = [field for field in fs_info['fields']
                                     if field != 'all_items_validated']
        return fieldsets

    def save_model(self, request, obj, form, change):
        """
        If this is a new item and not a change, save the new obj somewhere we
        can find it!

        This is necessary because we don't have a hook for this within
        add_view on either side of the super() call.
        """

        # Perpetuate the call to ModelAdmin.
        super(ContentAreaAdmin, self).save_model(request, obj, form, change)

        # If this was a freshly saved ContentArea, save it on the request.
        if not change:
            request.new_content_area_object = obj




from django.test import TestCase

from mock_project.test_app.models import MyArea

from .models import Video


class VideoTest(TestCase):
    def setUp(self):
        # Create an area with regular ordering (first item first).
        self.area = MyArea.objects.create(title="Blah")

    def test_youtube_validate_good_id(self):
        """
        Make sure a valid youtube ID can be validated.
        """

        # Create fake form data.
        fake_POST = {
            'content_area_id': self.area.pk,
            'content_area_ct': self.area.get_content_type().pk,
            'ordering': 1,
            'service': 'youtube',
            'video_id': "RnAAIHBCubM",
        }

        # Get the form using that fake POST data.
        form = Video().get_form(fake_POST)
        # Try to save it.
        form.save()

        if not form.instance.pk:
            self.fail("Couldn't validate an existing Vimeo ID.")

    def test_youtube_validate_invalid_id(self):
        """
        Make sure a valid youtube ID is rejected when invalid.
        """

        # Create fake form data.
        fake_POST = {
            'content_area_id': self.area.pk,
            'content_area_ct': self.area.get_content_type().pk,
            'ordering': 1,
            'service': 'youtube',
            'video_id': "abc123",
        }

        # Get the form using that fake POST data.
        form = Video().get_form(fake_POST)
        # Try to save it.
        try:
            form.save()
        except ValueError:
            pass
        else:
            self.fail("Form allowed an invalid Vimeo ID.")

    def test_vimeo_validate_good_id(self):
        """
        Make sure a valid vimeo ID can be validated.
        """

        # Create fake form data.
        fake_POST = {
            'content_area_id': self.area.pk,
            'content_area_ct': self.area.get_content_type().pk,
            'ordering': 1,
            'service': 'vimeo',
            'video_id': "59338758",
        }

        # Get the form using that fake POST data.
        form = Video().get_form(fake_POST)
        # Try to save it.
        form.save()

        if not form.instance.pk:
            self.fail("Couldn't validate an existing Vimeo ID.")

    def test_vimeo_validate_invalid_id(self):
        """
        Make sure a valid vimeo ID is rejected when invalid.
        """

        # Create fake form data.
        fake_POST = {
            'content_area_id': self.area.pk,
            'content_area_ct': self.area.get_content_type().pk,
            'ordering': 1,
            'service': 'vimeo',
            'video_id': "a1b2c3d4",
        }

        # Get the form using that fake POST data.
        form = Video().get_form(fake_POST)
        # Try to save it.
        try:
            form.save()
        except ValueError:
            pass
        else:
            self.fail("Form allowed an invalid Vimeo ID.")




from distutils.core import setup
import os
from setuptools import find_packages

# Find all packages here and below (should find ipanema and its children).
packages = find_packages()

setup(name='django-flexible-content',
      description="Allows granular, typed, extensible control over a page's content.",
      url='https://github.com/hjkelly/django-flexible-content',
      maintainer='Harrison Kelly',
      maintainer_email='hjkelly@gmail.com',
      version='0.9.1',
      packages=packages,
      include_package_data=True,
      install_requires=['Django >= 1.4',
                        # Required for the multi-table inheritance fanciness
                        # used by content items. Not really *required*, but it
                        # has a cool InheritanceManager/InheritanceQuerySet
                        # that makes queries more efficient.
                        'django-model-utils',
                        # This lets the Video type verify YouTube and Vimeo IDs
                        'requests'])



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db.models import get_model
from django.utils.translation import ugettext as _


def get_app_settings():
    """
    Load the settings and make sure it's not totally wrong.
    """

    app_settings = getattr(settings, 'FLEXIBLE_CONTENT', None)
    if app_settings is None:
        app_settings = {}

    # If the settings were defined as something other than None or a
    # dictionary, raise a stink.
    if not isinstance(app_settings, dict):
        message = _("Setting FLEXIBLE_CONTENT should be a dictionary; "
                    "instead, it was of type {}.".format(type(app_settings)))
        raise ImproperlyConfigured(message)

    return app_settings


def get_model_from_string(model_string):
    """
    Take a list of 'app.ModelName' strings and return a list of model classes.
    """

    # This Django function takes app names and model class names
    # separated by a dot/period.
    model = get_model(*model_string.split('.'))

    # If the model came back as None, it couldn't import it.
    if model is None:
        message = _("Model '{}' was defined in the settings for "
                    "django-flexible-content, but couldn't be loaded. Please "
                    "check your settings file!".format(model_string))
        raise ImportError(message)

    return model


def get_models_from_strings(model_strings):
    """
    Take a list of Python module strings and import those classes.
    """
    models = []

    # Loop through each model string.
    for s in model_strings:
        models.append(get_model_from_string(s))

    return models




"""
This file makes a way for random pieces of content to be assigned to any given
model instance.

For example, say you have a Page model that subclasses ContentArea. For a Page
with id 1, you create the following BaseItems, where the numbering is
indicative of order:
    1.  some text
    2.  an image with a heading and caption
    3.  some more text
    4.  a video thumbnail that, when clicked, opens up a video in a modal
    5.  even more text


video (or even an arbitrary, custom type of thing) to be assigned to a model
instance (such as a page, a blog post, a sidebar, etc).
"""

from django import forms
from django.conf import settings
from django.contrib.contenttypes.generic import (GenericForeignKey,
                                                 GenericRelation)
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.forms import ModelForm
from django.template.loader import render_to_string
from django.utils.translation import ugettext as _

from model_utils.managers import InheritanceManager

from .forms import BaseItemForm
from .utils import (get_app_settings,
                    get_model_from_string,
                    get_models_from_strings)


class BaseItemManager(InheritanceManager):
    def get_for_area(self, area):
        """
        Returns all items belonging to a given model instance, casted to their
        real types.

        Note that all queries (even the ones on the model) should go through
        this manager.
        """

        # Get the uncasted items tied to the given area.
        content_type = area.get_content_type()
        qs = BaseItem.objects.filter(content_area_ct=content_type,
                                     content_area_id=area.pk)
        # Order those items!
        qs = qs.order_by('ordering', 'pk')

        # Use InheritanceManager to automatically (and without extra queries)
        # downcast each item via the OneToOneField multi-table inhertiance
        # creates.
        return qs.select_subclasses()


class BaseItem(models.Model):
    """
    Offer functionality/fields that actual item subclasses need.
    """

    # Which model instance is this content part of?
    content_area_ct = models.ForeignKey(ContentType, related_name='+')
    content_area_id = models.PositiveIntegerField(db_index=True)
    content_area = GenericForeignKey(ct_field='content_area_ct',
                                     fk_field='content_area_id')

    # Relative to other items, where does this one belong?
    ordering = models.IntegerField(default=1, db_index=True)

    objects = BaseItemManager()

    class FlexibleContentInfo:
        description = _("This shouldn't ever be visible - it's just an "
                        "example of how you could specify a description in "
                        "an actual item.")
        type_slug = 'used-for-templates-and-the-like'

    class Meta:
        ordering = ['ordering', 'pk']

    def get_casted(self):
        """
        Ensure that this instance isn't merely a BaseItem, but casted to its
        true subclass type (per multi-table inheritance).
        """

        item = self
        # If this isn't a *subclass* of BaseItem, query for the subclass's item
        if type(item) not in BaseItem.__subclasses__():
            item = BaseItem.objects.all().select_subclasses().get(pk=item.pk)
        return item

    @classmethod
    def get_configured_types(cls):
        """
        Return a list of available item types.
        """

        # Do the settings define classes?
        configured_types = get_app_settings().get('ITEM_TYPES', None)

        # If there are types chosen, import them.
        if configured_types is not None:
            configured_types = get_models_from_strings(configured_types)
        # If not, grab the defaults.
        else:
            from .default_item_types.models import DEFAULT_TYPES
            configured_types = DEFAULT_TYPES

        return tuple(configured_types)

    def get_content_type(self):
        return ContentType.objects.get_for_model(self)

    def get_form_class(self):
        """
        You'll want to either specify a form class in your FlexibleContentInfo
        class, or override this and return the class straight away, like so:
            def get_form_class(self):
                from .forms import MyForm
                return MyForm
        """

        # Try to get an explicit form class first.
        form_class = getattr(self.FlexibleContentInfo, 'form_class', None)

        # If we didn't find one there, create a form on the fly.
        if form_class is None:
            name = 'AutoGenerated{}Form'.format(self.__class__.__name__)
            form_class_meta = type('Meta', (BaseItemForm.Meta,),
                                   {'model': self.__class__})
            form_class = type(name, (BaseItemForm,), {'Meta': form_class_meta})

        return form_class

    def get_form(self, *args, **kwargs):
        """
        Generate a form of this typeFor a given model instance, generate the
        form. Optionally take the data and file arguments that standard forms
        take, and pass those along.
        """

        # Manually set the kwargs to bind the form to this instance.
        if 'instance' not in kwargs:
            kwargs.update({'instance': self})

        # Generate the form using the args and kwargs we were given.
        return self.get_form_class()(*args, **kwargs)

    def get_form_template(self):
        from .admin import get_form_prefix
        return self.get_form(prefix=get_form_prefix())

    def get_rendered_content(self):
        """
        Use the template to render this instance's data, and cache it on the
        instance.
        """
        if getattr(self, '_rendered_content', None) is None:
            self._rendered_content = render_to_string(self.get_template_name(),
                                                      {'item': self})
        return self._rendered_content

    def get_template_name(self):
        return 'flexible-content/{}.html'.format(self.get_type_slug())

    def get_type_description(self):
        return getattr(self.FlexibleContentInfo, 'description', '')

    def get_type_name(self):
        name = getattr(self.FlexibleContentInfo, 'name', '')
        if not name:
            name = self._meta.verbose_name
        return name

    def get_type_slug(self):
        # Try to get an explicit slug first.
        slug = getattr(self.FlexibleContentInfo, 'type_slug', None)

        # If we didn't find one, guess it from the model name.
        if slug is None:
            slug = self.__class__.__name__.lower()

        return slug


class ContentArea(models.Model):
    rendered_content = None

    class Meta:
        abstract = True

    def delete(self, *args, **kwargs):
        """
        Ensure we delete content items too.
        """

        # What content items belong to this object?
        dead_items = list(self.items)

        # Delete the area.
        super(ContentArea, self).delete(*args, **kwargs)

        # Delete the area's old items, just in case.
        [i.delete() for i in dead_items]

    @property
    def items(self):
        return BaseItem.objects.get_for_area(self)

    def get_content_type(self):
        return ContentType.objects.get_for_model(self)

    def get_rendered_content(self):
        """
        Returns all content items rendered into a single string (likely HTML).
        """
        if self.rendered_content is None:
            rendered_items = [i.get_rendered_content() for i in self.items]
            self.rendered_content = '\n\n'.join(rendered_items)
        return self.rendered_content


class TemporaryArea(ContentArea):
    """
    Holds new/updated content items before we save the real area.

    This is necessary because we're overriding Django's views and it creates
    any messages and forms the redirect (or error response). We need to save
    the items before that happens, so it doesn't do all that junk when our
    content items are about to screw things up and change the proper response.
    """

    created = models.DateTimeField(auto_now_add=True)

    def migrate_items_to(self, real_area):
        """
        Take the items for this area and move them to the real area, then
        delete yourself!
        """

        # Figure out the two bits of data for the new area.
        real_area_data = {
            'content_area_ct': real_area.get_content_type().pk,
            'content_area_id': real_area.pk,
        }

        # Update the items for this temporary area.
        self.items.update(**real_area_data)

        # Delete the temporary area!
        self.delete()




from django.db import models
from django.utils.translation import ugettext as _

from flexible_content.models import ContentArea, BaseItem


class MyArea(ContentArea):
    title = models.CharField(max_length=50)


class MyItem(BaseItem):
    my_number = models.IntegerField()

    class FlexibleContentInfo:
        description = _("Hopefully this works!")
        type_slug = 'my-item'

    class Meta:
        verbose_name = _("My Item")
